import 'dart:async';

import 'package:flutter/material.dart';
import 'package:sendtrain/classes/activity_action.dart';
import 'package:sendtrain/classes/media.dart';
import 'package:sendtrain/models/activity_model.dart';
import 'package:sendtrain/widgets/activity_action_view.dart';
import 'package:sendtrain/widgets/media_card.dart';

class ActivityView extends StatefulWidget {
  ActivityView({super.key, required this.activity});

  ActivityModel activity;

  @override
  State<ActivityView> createState() => _ActivityViewState();
}

class _ActivityViewState extends State<ActivityView> {
  Timer? _periodicTimer;
  int _tickCount = 0;
  int _currentSet = 1;
  int _currentRep = 1;
  String _currentState = 'Rep';

  void _startPeriodicTimer() {
    _tickCount = widget.activity.actions[0].activityActionSet.rest ~/ 10000;
    const oneSecond = Duration(seconds: 1);
    _periodicTimer = Timer.periodic(oneSecond, (Timer timer) {
      setState(() {
        if (_tickCount == 0) {
          if (_currentSet ==
              widget.activity.actions[0].activityActionSet.total) {
            timer.cancel();
          }

          if (_currentRep ==
              widget.activity.actions[0].activityActionSet.reps
                  .amounts[_currentSet]) {
            _currentSet++;
            _currentRep = 1;
            _tickCount =
                widget.activity.actions[0].activityActionSet.rest ~/ 10000;
          } else {
            _currentRep++;
          }
        } else {
          _tickCount--;
        }
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    ActivityModel activity = widget.activity;

    var content = [
      AppBar(
        // surfaceTintColor: ThemeData.dark(useMaterial3: true).colorScheme.primary,
        // backgroundColor: ThemeData.dark(useMaterial3: true).colorScheme.primaryContainer,
        centerTitle: true,
        title: const Text('Activity', style: TextStyle(fontSize: 15)),
      ),
      Padding(
          padding:
              const EdgeInsets.only(left: 15, right: 20, top: 15, bottom: 10),
          child: Text(
              maxLines: 1,
              style: const TextStyle(fontSize: 25, fontWeight: FontWeight.bold),
              activity.title)),
      ActivityViewCategories(categories: activity.categories),
      Padding(
          padding:
              const EdgeInsets.only(top: 0, bottom: 20, left: 15, right: 15),
          child: Text(
              textAlign: TextAlign.left,
              style: const TextStyle(fontSize: 15),
              activity.description)),
      ActivityViewMedia(activity: activity),
      const Padding(
          padding: EdgeInsets.fromLTRB(15, 30, 0, 10),
          child: Text(
              textAlign: TextAlign.left,
              style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),
              'Actions:'))
    ];

    for (var action in activity.actions) {
      content.add(ActivityActionView(action: action));
    }

    // add bottom bar to manage activity
    content.add(Container(
        height: MediaQuery.sizeOf(context).height * .07,
        color: ThemeData.dark(useMaterial3: true).colorScheme.primaryContainer,
        child: Row(children: [
          Expanded(
              flex: 1,
              child: Flex(direction: Axis.horizontal, children: [
                IconButton(
                    iconSize: 30,
                    icon: const Icon(Icons.play_arrow_rounded),
                    onPressed: _startPeriodicTimer),
                IconButton(
                    iconSize: 30,
                    icon: const Icon(Icons.skip_next_rounded),
                    onPressed: () {})
              ])),
          Expanded(
            flex: 1,
            child: Text(
                style: const TextStyle(fontSize: 25),
                textAlign: TextAlign.center,
                '$_tickCount'),
          ),
          Expanded(
              flex: 1,
              child: Padding(
                  padding: const EdgeInsets.only(right: 10),
                  child: Text(
                      style: const TextStyle(fontSize: 15),
                      textAlign: TextAlign.right,
                      'Set: $_currentSet/3 \nRep: $_currentRep/5'))),
        ])));
    return Column(
        crossAxisAlignment: CrossAxisAlignment.start, children: content);
  }
}

class ActivityViewCategories extends StatelessWidget {
  const ActivityViewCategories({super.key, this.categories});

  final List<String>? categories;

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        Padding(
            padding: const EdgeInsets.only(bottom: 10),
            child: SizedBox(
                height: 40,
                child: ListView.builder(
                  shrinkWrap: true,
                  scrollDirection: Axis.horizontal,
                  padding: const EdgeInsets.fromLTRB(10, 0, 10, 0),
                  itemCount: categories?.length,
                  itemBuilder: (BuildContext context, int index) {
                    return Padding(
                        padding: const EdgeInsets.only(right: 5),
                        child: ActionChip(
                          visualDensity: VisualDensity.compact,
                          avatar: const Icon(Icons.check_circle_outline),
                          label: Text(maxLines: 1, '${categories?[index]}'),
                          onPressed: () {},
                        ));
                  },
                ))),
      ],
    );
  }
}

class ActivityViewMedia extends StatelessWidget {
  const ActivityViewMedia({super.key, this.activity});

  final ActivityModel? activity;

  @override
  Widget build(BuildContext context) {
    List<Media> media = [];

    for (ActivityAction action in activity!.actions) {
      if (action.media!.isNotEmpty) {
        media.addAll(action.media as Iterable<Media>);
      }
    }

    List<Widget> mediaCards =
        List.generate(media.length, (i) => MediaCard(media: media[i]));

    return Column(
      children: [
        SizedBox(
            width: double.infinity,
            height: 100,
            child: GridView.count(
                padding: const EdgeInsets.fromLTRB(15, 0, 0, 0),
                scrollDirection: Axis.horizontal,
                crossAxisSpacing: 5,
                mainAxisSpacing: 5,
                crossAxisCount: 1,
                children: mediaCards))
      ],
    );
  }
}
